/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Stack from './stack.js';

class Linter {
  constructor() {
    this.stack = new Stack();
  }

  lint(text) {
    while (this.stack.read()) {
      this.stack.pop();
    }

    const matchingBraces = { '(': ')', '[': ']', '{': '}' };

    for (const char of text) {
      if (matchingBraces[char]) {
        this.stack.push(char);
      } else if (Object.values(matchingBraces).includes(char)) {
        if (!this.stack.read()) {
          return `${char} does not have opening brace`;
        } else {
          const poppedOpeningBrace = this.stack.pop();

          if (char !== matchingBraces[poppedOpeningBrace]) {
            return `${char} has mismatched opening brace`;
          }
        }
      }
    }

    // If we get to the end of line, and the stack isn't empty:
    if (this.stack.read()) {
      return `${this.stack.read()} does not have closing brace`;
    }

    // Return true if line has no errors:
    return true;
  }
}
export default Linter;
