/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Stack from './stack.js';
import Linter from './linter.js';
import Queue from './queue.js';
import PrintManager from './print_manager.js';
import reverse from './solution.js';

function assertEqual(x, y) {
  if (x === y) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

// Basic Stack operations
console.log('Basic Stack operations');
const stack = new Stack();
stack.push('a');
assertEqual(stack.data[0], 'a'); // Push
stack.push('b');
assertEqual(stack.read(), 'b'); // Read
assertEqual(stack.pop(), 'b'); // Pop
assertEqual(stack.read(), 'a');
stack.pop();
assertEqual(stack.read(), null);
assertEqual(stack.pop(), null);

// JavaScript Linter
console.log('JavaScript linter');
const linter = new Linter();
const error1 = '(var x = 2;'; // Syntax Error 1
const error2 = 'var x = 2;)'; // Syntax Error 2
const error3 = '(var x = [1, 2, 3)];'; // Syntax Error 3
const correct = '(var x = [1, 2, 3])'; // Correct
assertEqual(linter.lint(error1), '( does not have closing brace');
assertEqual(linter.lint(error2), ') does not have opening brace');
assertEqual(linter.lint(error3), ') has mismatched opening brace');
assertEqual(linter.lint(correct), true);

// Basic Queue operations
console.log('Basic Queue operations');
const queue = new Queue();
queue.enqueue('a'); // Enqueue
queue.enqueue('b');
queue.enqueue('c');
assertEqual(queue.read(), 'a'); // Read
assertEqual(queue.dequeue(), 'a'); // Dequeue
assertEqual(queue.read(), 'b');
queue.dequeue();
queue.dequeue();
assertEqual(queue.read(), null);
assertEqual(queue.dequeue(), null);

// Print manager
console.log('Print manager');
const printManager = new PrintManager();
printManager.queuePrintJob('First Document');
printManager.queuePrintJob('Second Document');
printManager.queuePrintJob('Third Document');
printManager.run();

// Solution - reverse a string
console.log('Solution - reverse a string');
assertEqual(reverse('abcde'), 'edcba');
assertEqual(reverse('a'), 'a');
assertEqual(reverse(''), '');
