/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Node from './node.js';

class LinkedList {
  constructor(firstNode=null) {
    this.firstNode = firstNode;
  }
  read(index) {
    let currentNode = this.firstNode;
    let currentIndex = 0;

    while (currentIndex < index) {
      currentNode = currentNode.nextNode;
      currentIndex += 1;

      if (!currentNode) {
        return null;
      }
    }

    return currentNode.data;
  }
  search(value) {
    let currentNode = this.firstNode;
    let currentIndex = 0;

    while (true) {
      if (currentNode.data === value) {
        return currentIndex;
      }

      currentNode = currentNode.nextNode;

      if (!currentNode) {
        break;
      }

      currentIndex += 1;
    }

    return null;
  }
  append(value) {
    const newNode = new Node(value);

    if (!this.firstNode) {
      this.firstNode = newNode;
      return;
    }

    let currentNode = this.firstNode;

    while (true) {
      if (currentNode.nextNode) {
        currentNode = currentNode.nextNode;
      } else {
        currentNode.nextNode = newNode;
        break;
      }
    }
  }
  recursiveAppend(value, currentNode = null) {
    const newNode = new Node(value);

    if (!this.firstNode) {
      this.firstNode = newNode;
      return;
    }

    if (!currentNode) {
      currentNode = this.firstNode;
    }

    if (currentNode.nextNode) {
      this.recursiveAppend(value, currentNode.nextNode);
    } else {
      currentNode.nextNode = newNode;
    }
  }
  insert(index, value) {
    const newNode = new Node(value);

    if (index === 0) {
      newNode.nextNode = this.firstNode;
      this.firstNode = newNode;
      return;
    }

    let currentNode = this.firstNode;
    let currentIndex = 0;

    while (currentIndex < (index - 1)) {
      currentNode = currentNode.nextNode;
      currentIndex += 1;
    }

    newNode.nextNode = currentNode.nextNode;

    currentNode.nextNode = newNode;
  }
  delete(index) {
    if (index === 0) {
      this.firstNode = this.firstNode.nextNode;
      return;
    }

    let currentNode = this.firstNode;
    let currentIndex = 0;

    while (currentIndex < (index - 1)) {
      currentNode = currentNode.nextNode;
      currentIndex += 1;
    }

    const nodeAfterDeletedNode = currentNode.nextNode.nextNode;
    currentNode.nextNode = nodeAfterDeletedNode;
  }

  printList() {
    let currentNode = this.firstNode;

    while (currentNode) {
      console.log(currentNode.data);
      currentNode = currentNode.nextNode;
    }
  }

  last() {
    let currentNode = this.firstNode;

    while (currentNode.nextNode) {
      currentNode = currentNode.nextNode;
    }

    return currentNode.data;
  }

  recursiveLast(currentNode = null) {
    if (!currentNode) {
      currentNode = this.firstNode;
    }

    if (currentNode.nextNode) {
      return this.recursiveLast(currentNode.nextNode);
    } else {
      return currentNode.data;
    }
  }

  reverse() {
    let previousNode = null;
    let currentNode = this.firstNode;

    while (currentNode) {
      const nextNode = currentNode.nextNode;
      currentNode.nextNode = previousNode;

      previousNode = currentNode;
      currentNode = nextNode;
    }

    this.firstNode = previousNode;
  }
}
export default LinkedList;
