/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Node from './node.js';
import LinkedList from './linked_list.js';
import deleteNode from './solution5.js';
import DoublyLinkedList from './doubly_linked_list.js';
import Queue from './queue.js';

function assertEqual(x, y) {
  if (x === y) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

// Node
console.log('Node');
const node1 = new Node('once');
const node2 = new Node('upon');
const node3 = new Node('a');
const node4 = new Node('time');
node1.nextNode = node2;
node2.nextNode = node3;
node3.nextNode = node4;
assertEqual(node1.data, 'once');
assertEqual(node1.nextNode, node2);
assertEqual(node1.nextNode.nextNode.nextNode, node4);

// Linked List - Basic
console.log('Linked List - Basic');
const list = new LinkedList(node1);
assertEqual(list.firstNode, node1);
assertEqual(list.firstNode.nextNode, node2);

// Linked List - Read
console.log('Linked List - Read');
assertEqual(list.read(2), 'a');
assertEqual(list.read(6), null);

// Linked List - Search
console.log('Linked List - Search');
assertEqual(list.search('a'), 2);
assertEqual(list.search('happily'), null);

// Linked List - Insert
console.log('Linked List - Insert');
list.insert(4, 'there');
assertEqual(list.search('there'), 4);
list.insert(0, 'Hello!');
assertEqual(list.search('Hello!'), 0);

// Linked List - Delete
console.log('Linked List - Delete');
list.delete(0);
assertEqual(list.search('once'), 0);
list.delete(4);
assertEqual(list.read(4), null);
assertEqual(list.read(3), 'time');

// // Linked List - Append
console.log('Linked List - Append');
let newList = new LinkedList();
newList.append(1);
newList.append(2);
assertEqual(newList.read(0), 1);
assertEqual(newList.read(1), 2);
assertEqual(newList.read(2), null);

// // Linked List - Recursive Append
console.log('Linked List - Recursive Append');
newList = new LinkedList();
newList.recursiveAppend(1);
newList.recursiveAppend(2);
newList.recursiveAppend(3);
assertEqual(newList.read(0), 1);
assertEqual(newList.read(1), 2);
assertEqual(newList.read(2), 3);
assertEqual(newList.read(3), null);

// Doubly Linked List - Append
console.log('Doubly Linked List - Append');
const dlist = new DoublyLinkedList();
dlist.append('A');
assertEqual(dlist.firstNode.data, 'A');
assertEqual(dlist.lastNode.data, 'A');
dlist.append('B');
dlist.append('C');
assertEqual(dlist.firstNode.data, 'A');
assertEqual(dlist.lastNode.data, 'C');

// Doubly Linked List - pop head
console.log('Doubly Linked List - pop head');
let poppedNode = dlist.popHead();
assertEqual(poppedNode.data, 'A');
assertEqual(dlist.firstNode.data, 'B');
assertEqual(dlist.lastNode.data, 'C');
assertEqual(dlist.firstNode.previousNode, null);

// Queue
console.log('Queue');
const queue = new Queue();
assertEqual(queue.read(), null);
queue.enqueue('A');
queue.enqueue('B');
assertEqual(queue.read(), 'A');
poppedNode = queue.dequeue();
assertEqual(poppedNode, 'A');
assertEqual(queue.read(), 'B');

// Solution 1 : should print once upon a time
console.log('Solution 1');
list.printList();

// Solution 2 : should print: B A
console.log('Solution 2');
dlist.reversePrint();

// Solution 3
console.log('Solution 3');
assertEqual(list.last(), 'time');

// Solution 3b
console.log('Solution 3b');
assertEqual(list.recursiveLast(), 'time');

// Solution 4
console.log('Solution 4');
list.reverse();
assertEqual(list.last(), 'once');
assertEqual(list.firstNode.data, 'time');
list.reverse(); // Preparing for next test

// // Solution 5
console.log('Solution 5');
deleteNode(node3);
assertEqual(list.read(0), 'once');
assertEqual(list.read(1), 'upon');
assertEqual(list.read(2), 'time');
