/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import TreeNode from './tree_node.js';
import search from './search.js';
import insert from './insert.js';
import deleteNode from './delete.js';
import { traverseAndPrint as traverseAndPrint1 } from './preorder_traversal.js';
import { traverseAndPrint as traverseAndPrint2 } from './in_order_traversal.js';
import { traverseAndPrint as traverseAndPrint3 } from './postorder_traversal.js';
import max from './solution3.js';

function assertEqual(x, y) {
  if (x === y) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

// Tree Node - Basic
console.log('BST basic');
const node1 = new TreeNode(25);
const node2 = new TreeNode(75);
let root = new TreeNode(50, node1, node2);
assertEqual(root.value, 50);
assertEqual(root.rightChild.value, 75);
assertEqual(root.leftChild.value, 25);

// BST search
console.log('BST search');
assertEqual(search(50, root), root);
assertEqual(search(75, root), node2);
assertEqual(search(25, root), node1);
assertEqual(search(100, root), null);
assertEqual(search(0, root), null);

// BST insertion
console.log('BST insertion');
root = new TreeNode(50);
insert(25, root);
insert(75, root);
insert(60, root);
insert(100, root);
insert(90, root);
insert(5, root);
insert(40, root);
assertEqual(search(50, root).value, 50);
assertEqual(search(25, root).value, 25);
assertEqual(search(75, root).value, 75);
assertEqual(search(60, root).value, 60);
assertEqual(search(100, root).value, 100);
assertEqual(search(90, root).value, 90);
assertEqual(search(5, root).value, 5);
assertEqual(search(40, root).value, 40);

// BST - delete
root = new TreeNode(50);
insert(25, root);
insert(75, root);
insert(10, root);
insert(33, root);
insert(4, root);
insert(11, root);
insert(30, root);
insert(40, root);
insert(56, root);
insert(89, root);
insert(52, root);
insert(61, root);
insert(82, root);
insert(95, root);

// Delete node from bottom level
console.log('Delete node from bottom level');
assertEqual(search(4, root).value, 4);
deleteNode(4, root);
deleteNode(44, root);
assertEqual(search(4, root), null);

// Delete node with a right child
console.log('Delete node with only right child');
assertEqual(search(10, root).value, 10);
deleteNode(10, root);
assertEqual(search(10, root), null);
assertEqual(search(25, root).leftChild.value, 11);

// Delete node with a left child
console.log('Delete node with only left child');
deleteNode(40, root);
deleteNode(33, root);
assertEqual(search(33, root), null);
assertEqual(search(25, root).rightChild.value, 30);

// Delete root node of tree when successor node has no children
console.log('Delete root node of tree when successor node has no children');
assertEqual(search(50, root).value, 50);
let newRoot = deleteNode(50, root);
assertEqual(newRoot.value, 52);
assertEqual(search(50, root), null);
assertEqual(search(56, root).leftChild, null);
assertEqual(newRoot.rightChild.value, 75);

// Delete root node of tree when successor node has a right child
console.log('Delete root node of complex tree when successor node has a right child');
newRoot = deleteNode(52, root);
assertEqual(newRoot.value, 56);
assertEqual(search(52, root), null);
assertEqual(search(75, root).leftChild.value, 61);

// Delete node of tree when successor node is the right child
console.log('Delete node of tree when successor node is the right child');
root = new TreeNode(50);
insert(25, root);
insert(75, root);
insert(89, root);
insert(90, root);
insert(95, root);
newRoot = deleteNode(50, root);
assertEqual(newRoot.value, 75);
assertEqual(newRoot.rightChild.value, 89);

// Delete root with one child
console.log('Delete root with one child');
root = new TreeNode(50);
insert(75, root);
insert(89, root);
insert(90, root);
insert(95, root);
newRoot = deleteNode(50, root);
assertEqual(newRoot.value, 75);
assertEqual(newRoot.rightChild.value, 89);

// Preorder traversal
console.log('Preorder traversal');
root = new TreeNode(5);
insert(2, root);
insert(7, root);
insert(1, root);
insert(3, root);
insert(6, root);
insert(8, root);
traverseAndPrint1(root);

// Inorder traversal
console.log('Inorder traversal');
traverseAndPrint2(root);

// Postorder traversal
console.log('Postorder traversal');
traverseAndPrint3(root);

// Solution 3
console.log('Solution 3');
assertEqual(max(root), 8);
