/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Trie from './trie.js';

function assertEqual(x, y) {
  if (x === y) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

function assertObjectEqual(x, y) {
  if (JSON.stringify(x) === JSON.stringify(y)) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

// Print children to console
console.log('Insert and Search');
const tr = new Trie();
tr.insert('a');
tr.insert('b');
tr.insert('c');
console.log(tr.root.children);

// Insert and Search
console.log('Insert and Search');
const t = new Trie();
t.insert('cat');
t.insert('dog');
t.insert('catnip');
t.insert('catnap');
t.insert('category');
console.log(t.root.children);
assertObjectEqual(t.search('catnap').children, { '*': null });
assertEqual(t.search('catnax'), null);

// Collect all words
console.log('Collect all words');
const words = t.collectAllWords([]);
assertEqual(words.includes('cat'), true);
assertEqual(words.includes('dog'), true);
assertEqual(words.includes('catnip'), true);
assertEqual(words.includes('catnap'), true);
assertEqual(words.includes('category'), true);

// Autocomplete
console.log('Autocomplete');
const suggestions = t.autocomplete('ca');
assertEqual(suggestions.includes('t'), true);
assertEqual(suggestions.includes('tnip'), true);
assertEqual(suggestions.includes('tnap'), true);
assertEqual(suggestions.includes('tegory'), true);
assertEqual(suggestions.includes('dog'), false);

// Autocorrect
console.log('Autocorrect');
assertEqual(t.autocorrect('catnar'), 'catnap');
assertEqual(t.autocorrect('caxasfdij'), 'cat');

// Traverse
console.log('Traverse');
t.traverse();
