/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import TrieNode from './trie_node.js';

class Trie {
  constructor() {
    this.root = new TrieNode();
  }
  search(word) {
    let currentNode = this.root;

    for (const char of word) {
      if (currentNode.children[char]) {
        currentNode = currentNode.children[char];
      } else {
        return null;
      }
    }

    return currentNode;
  }
  insert(word) {
    let currentNode = this.root;

    for (const char of word) {
      if (currentNode.children[char]) {
        currentNode = currentNode.children[char];
      } else {
        const newNode = new TrieNode();
        currentNode.children[char] = newNode;
        currentNode = newNode;
      }
    }

    currentNode.children['*'] = null;
  }
  collectAllWords(words, node=null, word='') {
    const currentNode = node || this.root;

    for (const [key, childNode] of Object.entries(currentNode.children)) {
      if (key === '*') {
        words.push(word);
      } else {
        this.collectAllWords(words, childNode, word + key);
      }
    }

    return words;
  }
  autocomplete(prefix) {
    const currentNode = this.search(prefix);

    if (!currentNode) { return null; }

    return this.collectAllWords([], currentNode);
  }
  traverse(node = null) {
    const currentNode = node || this.root;

    for (const [key, childNode] of Object.entries(currentNode.children)) {
      console.log(key);

      if (key !== '*') {
        this.traverse(childNode);
      }
    }
  }
  autocorrect(word) {
    let currentNode = this.root;
    let wordFoundSoFar = '';

    for (const char of word) {
      if (currentNode.children[char]) {
        wordFoundSoFar += char;
        currentNode = currentNode.children[char];
      } else {
        return wordFoundSoFar + this.collectAllWords([], currentNode)[0];
      }
    }

    return word;
  }
}

export default Trie;
