/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
function dijkstraShortestPath(startingCity, finalDestination) {
  const cheapestPricesTable = {};
  const cheapestPreviousStopoverCityTable = {};
  let unvisitedCities = [startingCity];
  const visitedCities = {};

  cheapestPricesTable[startingCity.name] = 0;
  let currentCity = startingCity;

  while (unvisitedCities.length > 0) {
    visitedCities[currentCity.name] = true;
    unvisitedCities = unvisitedCities.filter((city) => city !== currentCity);

    for (const adjacentCity of currentCity.routes.keys()) {
      const price = currentCity.routes.get(adjacentCity);

      if (!visitedCities[adjacentCity.name] && !unvisitedCities[adjacentCity]) {
        unvisitedCities.push(adjacentCity);
      }

      const priceThroughCurrentCity = 
        (cheapestPricesTable[currentCity.name] + price);

      if (!cheapestPricesTable[adjacentCity.name] || priceThroughCurrentCity
              < cheapestPricesTable[adjacentCity.name]) {
        cheapestPricesTable[adjacentCity.name] = priceThroughCurrentCity;
        cheapestPreviousStopoverCityTable[adjacentCity.name] = currentCity.name;
      }
    }

    let cheapestPrice = Infinity;

    for (const city of unvisitedCities) {
      if (cheapestPricesTable[city.name] < cheapestPrice) {
        cheapestPrice = cheapestPricesTable[city.name];
        currentCity = city;
      }
    }
  }

  const shortestPath = [];
  let currentCityName = finalDestination.name;

  while (currentCityName) {
    shortestPath.unshift(currentCityName);
    currentCityName = cheapestPreviousStopoverCityTable[currentCityName];
  }

  return shortestPath;
}
export default dijkstraShortestPath;
