/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
import Vertex from './vertex.js';
import { Vertex as UndirectedVertex } from './undirected_graph_vertex.js';
import dfsTraverse from './dfs_traverse.js';
import bfsTraverse from './bfs_traverse.js';
import dfs from './dfs.js';
import bfs from './solution4.js';
import findShortestPath from './solution5.js';
import WeightedGraphVertex from './weighted_graph_vertex.js';
import City from './city.js';
import dijkstraShortestPath from './dijkstra.js';

function assertEqual(x, y) {
  if (x === y) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

function assertArrayEqual(x, y) {
  if (JSON.stringify(x) === JSON.stringify(y)) {
    console.log('.');
  } else {
    console.log('FAIL');
  }
}

// Vertex - Directed Graph - Add adjacent vertex
console.log('Vertex');
let alice = new Vertex('alice');
let bob = new Vertex('bob');
const cynthia = new Vertex('cynthia');

alice.addAdjacentVertex(bob);
alice.addAdjacentVertex(cynthia);
assertEqual(alice.adjacentVertices[0].value, 'bob');
assertEqual(alice.adjacentVertices[1].value, 'cynthia');

// Vertex - Undirected Graph - Add adjacent vertex
console.log('Undirected Vertex');
alice = new UndirectedVertex('alice');
bob = new UndirectedVertex('bob');
alice.addAdjacentVertex(bob);
assertEqual(alice.adjacentVertices[0].value, 'bob');
assertEqual(bob.adjacentVertices[0].value, 'alice');

// Depth-first traverse
alice = new Vertex('Alice');
bob = new Vertex('Bob');
const candy = new Vertex('Candy');
const derek = new Vertex('Derek');
const elaine = new Vertex('Elaine');
const fred = new Vertex('Fred');
const gina = new Vertex('Gina');
const helen = new Vertex('Helen');
const irena = new Vertex('Irena');

alice.addAdjacentVertex(bob);
alice.addAdjacentVertex(candy);
alice.addAdjacentVertex(derek);
alice.addAdjacentVertex(elaine);
bob.addAdjacentVertex(fred);
fred.addAdjacentVertex(helen);
derek.addAdjacentVertex(gina);
gina.addAdjacentVertex(irena);
bob.addAdjacentVertex(alice);
candy.addAdjacentVertex(alice);
derek.addAdjacentVertex(alice);
elaine.addAdjacentVertex(alice);
fred.addAdjacentVertex(bob);
helen.addAdjacentVertex(fred);
gina.addAdjacentVertex(derek);
irena.addAdjacentVertex(gina);

console.log('DFS Traverse');
dfsTraverse(alice);

// BFS traverse
console.log('BFS Traverse');
bfsTraverse(alice);

// DFS
console.log('DFS');
assertEqual(dfs(alice, 'Alice'), alice);
assertEqual(dfs(alice, 'Gina'), gina);
assertEqual(dfs(alice, 'Jorge'), null);

// Solution 4 (BFS Search)
console.log('Solution 4');
assertEqual(bfs(alice, 'Alice'), alice);
assertEqual(bfs(alice, 'Gina'), gina);
assertEqual(bfs(alice, 'Jorge'), null);

// Solution 5 (BFS Search)
console.log('Solution 5');
const idris = new Vertex('Idris');
const talia = new Vertex('Talia');
const ken = new Vertex('Ken');
const marco = new Vertex('Marco');
const sasha = new Vertex('Sasha');
const lina = new Vertex('Lina');
const kamil = new Vertex('Kamil');
idris.addAdjacentVertex(talia);
talia.addAdjacentVertex(idris);
talia.addAdjacentVertex(ken);
ken.addAdjacentVertex(talia);
ken.addAdjacentVertex(marco);
marco.addAdjacentVertex(ken);
marco.addAdjacentVertex(sasha);
sasha.addAdjacentVertex(marco);
sasha.addAdjacentVertex(lina);
lina.addAdjacentVertex(sasha);
lina.addAdjacentVertex(kamil);
kamil.addAdjacentVertex(lina);
kamil.addAdjacentVertex(idris);
idris.addAdjacentVertex(kamil);

assertArrayEqual(findShortestPath(idris, lina, {}), ['Idris', 'Kamil', 'Lina']);

// Weighted graph vertex
console.log('Weighted graph vertex');
const a = new WeightedGraphVertex('A');
const b = new WeightedGraphVertex('B');
a.addAdjacentVertex(b, 100);
assertEqual(a.adjacentVertices.get(b), 100);

// City-specific vertex
console.log('City-specific vertex');
let chicago = new City('Chicago');
let denver = new City('Denver');
chicago.addRoute(denver, 100);
// for (const route of chicago.routes.keys()) {
//   console.log(route);
// }
assertEqual(chicago.routes.get(denver), 100);

// Dijkstra's Algorithm
console.log('Dijkstras Algorithm');
const atlanta = new City('Atlanta');
const boston = new City('Boston');
chicago = new City('Chicago');
denver = new City('Denver');
const elPaso = new City('El Paso');

atlanta.addRoute(boston, 100);
atlanta.addRoute(denver, 160);
boston.addRoute(chicago, 120);
boston.addRoute(denver, 180);
chicago.addRoute(elPaso, 80);
denver.addRoute(chicago, 40);
denver.addRoute(elPaso, 140);
elPaso.addRoute(boston, 100);

// console.log(dijkstraShortestPath(atlanta, elPaso));
assertArrayEqual(dijkstraShortestPath(atlanta, elPaso), ['Atlanta', 'Denver', 'Chicago', 'El Paso']);
