/***
 * Excerpted from "A Common-Sense Guide to Data Structures and Algorithms, JavaScript Edition",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit https://pragprog.com/titles/jwjavascript for more book information.
***/
'use strict';

var GetIntrinsic = require('get-intrinsic');

var $TypeError = GetIntrinsic('%TypeError%');

var callBound = require('call-bind/callBound');
var has = require('has');

var $charCodeAt = callBound('String.prototype.charCodeAt');
var $toUpperCase = callBound('String.prototype.toUpperCase');

var Type = require('./Type');

var assertRecord = require('../helpers/assertRecord');
var caseFolding = require('../helpers/caseFolding');

// https://262.ecma-international.org/14.0/#sec-runtime-semantics-canonicalize-ch

module.exports = function Canonicalize(rer, ch) {
	assertRecord(Type, 'RegExp Record', 'rer', rer);
	if (Type(ch) !== 'String') {
		throw new $TypeError('Assertion failed: `ch` must be a character');
	}

	if (rer['[[Unicode]]'] && rer['[[IgnoreCase]]']) { // step 1
		if (has(caseFolding.C, ch)) {
			return caseFolding.C[ch];
		}
		if (has(caseFolding.S, ch)) {
			return caseFolding.S[ch];
		}
		return ch; // step 1.b
	}

	if (!rer['[[IgnoreCase]]']) {
		return ch; // step 2
	}

	var u = $toUpperCase(ch); // step 5

	if (u.length !== 1) {
		return ch; // step 7
	}

	var cu = u; // step 8

	if ($charCodeAt(ch, 0) >= 128 && $charCodeAt(cu, 0) < 128) {
		return ch; // step 9
	}

	return cu; // step 10
};
